/***********************************************************************

	Command:  FILES -- A General Purpose File Manager

	Programmed by Kang, Hsin-Min.
	First edition in 1995 for TCAM/TurboCAD (TwinCAD's DOS version)
	Revised at Sep 13, 1997 for TwinCAD as standard command:

	    1. Add [Help] button for help invocation in Windows.
	    2. Change programming style for multiple language support.

	Revised at Apr 22, 1998 Include Japenese Text in SJIS codes
	Revised at Aug 05, 1998 Modify Japenese Messages.

	Copyright (c) 1995, 1997, 1998 TCAM Development House, Taipei

******************************************************************/

#define strlen(s1)		len(s1)
#define ESC		-1
#define QUIT		0
#define LoopForever	while(1)

int	IsWin;
int	bList;
int	bCopy;
int	bRename;
int	bErase;
int	bMove;
int	bMCopy;
int	bExit;
int	bHelp;

char	szBtn[12][16];	 // For all common buttons
char	szText[20][40];  // Our display text buffer, see SetMsgText()
			// at the end of this file

COMMAND Files()
{
	int	fnc;

	SetMsgText();
	MainDiaWnd();		// Open the main dialogue window
	LoopForever {
	    fnc=wnd_gbtn(0x1000);	// Return any keypress
	    switch(fnc) {
	      case QUIT:
	      case ESC:
	      case bExit:
		wndclose();
		exit(0);
		break;
	      case bList:	DoListFile();	continue;
	      case bCopy:	DoCopyFile();	continue;
	      case bRename:	DoRename();	continue;
	      case bErase:	DoDelete();	continue;
	      case bMove:	DoMoveFile();	continue;
	      case bMCopy:	DoCopyDir();	continue;
	      case bHelp:	show_help("","FILES");  continue;
	    }
	    if ( IsWin ) continue;
	    switch(fnc&0xff) {	// Check Keyboard codes for DOS version
	      case 'L': case 'l':       DoListFile();   continue;
	      case 'D': case 'd':       DoCopyFile();   continue;
	      case 'R': case 'r':       DoRename();     continue;
	      case 'E': case 'e':       DoDelete();     continue;
	      case 'M': case 'm':       DoMoveFile();   continue;
	      case 'C': case 'c':       DoCopyDir();    continue;
	      case 'H': case 'h':       Show_help("","FILES");  continue;
	      case 'X': case 'x':
		wndclose();
		exit(0);
	    }
	}
}

DoListFile()
{
	getfile("*.*","*",0,szText[1]/*"Browse Directory Files"*/);
}

DoCopyFile()
{
	char	SrcFile[64];	// Source file
	char	DstFile[64];	// Destination file
	long	nbytes; 	// Number of Bytes Copied

	SrcFile = getfile("*.*","*",0,szText[2]);
	     /*"Select Single Source File to Copy"*/

	if ( !strlen(SrcFile) ) return 0;

	DstFile = getfile(SrcFile,"*",1,szText[3]);
	     /*"Enter the Destination Filename"*/

	if ( !strlen(DstFile) ) return 0;

	CopyOneFile(SrcFile,DstFile);
	beep();
}

DoRename()
{
	char	SrcFile[64];	// Source file
	char	DstFile[64];	// Destination file
	int	i;

	SrcFile = getfile("*.*","*",0,szText[4]);
	      /*"Select File to Rename"*/
	if ( !strlen(SrcFile) ) return 0;

	DstFile = getfile(SrcFile,"*",1,szText[5]);
		/*"Enter the New Filename"*/
	if ( !strlen(DstFile) ) return 0;

	printf("\nRename %s to %s",SrcFile,DstFile);
	if ( takepath(SrcFile) != takepath(DstFile) )
	    i = MoveOneFile(Srcfile,DstFile);
	else
	    i = rename(SrcFile,DstFile);
	switch(i) {
	  case 2:	printf("\n## File not found!"); break;
	  case 3:	printf("\n## Path not found!"); break;
	  case 5:	printf("\n## Access Denied!");  break;
	  case 0x11:	printf("\n## Not the same device!"); break;
	}
	beep();
}

DoDelete()
{
	char	ListFile[64];	// File Name List File
	char	SrcFile[65];	// File to erase
	int	hdl;
	int	state=0;

	ListFile = getfile("*.*","*",0x8000,szText[6]);
		/*"Select Files to Erase"*/
	if ( !strlen(ListFile) )  return 0;
	if ( index(ListFile,".FNL") ) {         // In case old version!
	    hdl = fopen(ListFile,"r");
	    while(fgets(SrcFile,64,hdl)>0) {
		if ( state != 2 )
		    state = ConfirmFile(szText[11]+SrcFile+" ?");
		if ( state ) {
		    printf("\nErase File: %s",SrcFile);
		    if ( remove(SrcFile) == 5 )
			printf(" ## Access Denied!");
		}
	    }
	}
	remove(ListFile);
	beep();
}

DoMoveFile()
{
	char	ListFile[64];	// File Name List File
	char	SrcFile[65];	// File to Move
	char	DstFile[65];	// Where to Move
	int	hdl;
	int	state=0;

	if ( OldVersion() ) return 0;

	ListFile = getfile("*.*","*",0x8000,szText[7]);
	/*"Select Files to Move" */
	if ( !strlen(ListFile) )  return 0;
	DstFile = upper(getpath(takepath(ListFile),0,szText[8]));
	/*"Select Directory to Move Files to"*/
	if ( strlen(DstFile) )	{
	    hdl = fopen(ListFile,"r");
	    while(fgets(SrcFile,64,hdl)>0) {
		if ( takepath(SrcFile)==DstFile )
		    break;
		if ( state != 2 )
		    state = ConfirmFile(szText[12]+SrcFile+" ?");
		if ( state ) {
		    printf("\nMove File: %s",SrcFile);
		    if ( MoveOneFile(SrcFile,
			      fullname(takename(SrcFile),DstFile)) == 5 )
			printf(" ## Access Denied!");
		}
	    }
	}
	remove(ListFile);
	beep();
}

DoCopyDir()
{
	char	ListFile[64];	// File Name List File
	char	SrcFile[65];	// File to Move
	char	DstFile[65];	// Where to Move
	int	hdl;
	int	state=0;
	long	nbytes, tnbytes;

	if ( OldVersion() ) return 0;

	ListFile = getfile("*.*","*",0x8000,szText[9]);
	/*"Select Files to Copy"*/
	if ( !strlen(ListFile) )  return 0;
	DstFile = upper(getpath(takepath(ListFile),0,szText[10]));
	/*"Select Directory to Copy Files to"*/
	if ( strlen(DstFile) ) {
	    hdl = fopen(ListFile,"r");
	    while(fgets(SrcFile,64,hdl)>0) {
		if ( takepath(SrcFile)==DstFile )
		    break;
		if ( state != 2 )
		    state = ConfirmFile(szText[13]+SrcFile+" ?");
		if ( state ) {
		    if ( CopyOneFile(SrcFile,
			      fullname(takename(SrcFile),DstFile)) )
			break;
		}
	    }
	}
	remove(ListFile);
	beep();
}

ConfirmFile(char Message[])
{
	int	wnd[]={13,17,5,75,0x371e};
	int	i;

	i = strlen(Message)+8;
	if ( i < 36 ) i = 36;
	if ( i > 80 )
	   i = 80;
	i /= 2;
	wnd[2] = 40-i;
	wnd[3] = 40+i-1;
	beep();
	wndopen(wnd);
	wnd_loct(1,2);
	wnd_puts(Message,0x61e);

	WndButton(3,  i-16, 0x108,szBtn[9]/*" NO  "*/);  // 0
	WndButton(3,  i-4,  0x108,szBtn[8]/*" Yes "*/);  // 1
	WndButton(3,  i+8,  0x108,szBtn[10]/*" All "*/); // 2

	LoopForever {
	   i = wnd_gbtn(0x1000);
	   switch(i&0xff) {
	     case 0:	case 1:    case 2:	break;
	     case 'N':  case 'n':  i = 0;       break;
	     case 'Y':  case 'y':  i = 1;       break;
	     case 'A':  case 'a':  i = 2;       break;
	     default:	continue;
	   }
	   wndclose();
	   return i;
	}
}

OldVersion()
{
	if ( !identify("getpath") ) {
	    printf("\n## Your Version is too old to have the 'getpath()' function support!");
	    beep();
	    return 1;
	}
	return 0;
}

MoveOneFile(char SrcFile[],char DstFile[])
{
	if ( copyfile(SrcFile,DstFile) < 0 )
	    return 5;
	return remove(SrcFile);
}

CopyOneFile(char SrcFile[],char DstFile[])
{
	long	nbytes;
	if ( identify("copyfile") ) {
	    printf("\nCopy %s to %s",SrcFile,DstFile);
	    nbytes = copyfile(SrcFile,DstFile);
	    if ( nbytes > 0 )
		printf("\nOperation Succeeded, %ld bytes copied.",nbytes);
	    else if ( nbytes == -1L ) {
		printf("\n## Source File Error!");      return 1;
	    }
	    else if ( nbytes == -2L ) {
		printf("\n## Destination File Error!"); return 1;
	    }
	}
	else
	    command("DOS COPY \@SrcFile \@DstFile^MEXIT^M");
	return 0;
}

MainDiaWnd()
{
	int	wnd[]={10,18,13,63,0x372f};
	int	rBase, cBase;

	SetMsgText();

	if ( IsWin )
		wnd[4] = 0x3271;

	wndopen(wnd,szText[0]/*"File Utilities"*/,0x321e,0x0000);

	rBase = 1;
	cBase = 2;
	bList	= WndButton(rBase,  cBase,   0x10e,szBtn[0]/*" List files "*/);
	bCopy	= WndButton(rBase,  cBase+16,0x10e,szBtn[1]/*" Duplicate  "*/);
	bRename = WndButton(rBase,  cBase+32,0x10e,szBtn[2]/*" Rename file"*/);
	bErase	= WndButton(rBase+2,cBase,   0x10e,szBtn[3]/*" Erase files"*/);
	bMove	= WndButton(rBase+2,cBase+16,0x10e,szBtn[4]/*" Move files "*/);
	bMCopy	= WndButton(rBase+2,cBase+32,0x10e,szBtn[5]/*" Copy files "*/);

	if ( IsWin ) {
		bExit = WndButton(rBase+4,cBase+10,0x108,szBtn[6]/*"  Exit  "*/);
		bHelp = WndButton(rBase+4,cBase+26,0x108,szBtn[7]/*"  Help  "*/);
	}
	else {	// No help in DOS version!
		bExit = WndButton(rBase+4,cBase+20,0x108,szBtn[6]/*"  Exit  "*/);
	}
}
//
//	Some useful routine supports
//
int	nSupport=identify("wnd_resize");
//
int	WndButton(int r,int c, int bsize, char text[])
{
	int	i;
	if ( nSupport ) 	// Having direct support ?
		return wnd_button(r,c,bsize,0x1277,text);
	wnd_loct(r,c);
	i = wnd_field(0,IsWin?0x1277:0x370,bsize);
	wnd_puts(text,0x270);
	return i;
}


char	szAsciiText[][40]={	    // Default English Display Text
/*  0 */   "File Utilities",
/*  1 */   "Browse Directory Files",
/*  2 */   "Select Single Source File to Copy",
/*  3 */   "Enter the Destination Filename",
/*  4 */   "Select File to Rename",
/*  5 */   "Enter the New Filename",
/*  6 */   "Select Files to Erase",
/*  7 */   "Select Files to Move",
/*  8 */   "Select Directory to Move Files to",
/*  9 */   "Select Files to Copy",
/* 10 */   "Select Directory to Copy Files to",
/* 11 */   "Erase ",
/* 12 */   "Move ",
/* 13 */   "Copy ",
};

char	szBig5Text[][40]={	   // Default Chinese Big-5 Display Text
/*  0 */   "   z  @",
/*  1 */   "     ",
/*  2 */   "   n  s   ",
/*  3 */   "  J n  s    ",
/*  4 */   "   n  W   ",
/*  5 */   "  J s   W  |",
/*  6 */   "   n R    ",
/*  7 */   "   n h    ",
/*  8 */   "  w h    a  |",
/*  9 */   "   n     ",
/* 10 */   "  w     a  |",
/* 11 */   "zTwnR ",
/* 12 */   "zTwnh ",
/* 13 */   "zTwn ",
};

char	szGB2312Text[][40]={	// Default Chinese GB2312 Display Text
/*  0 */   "     ",
/*  1 */   "  Ŀ ¼  ",
/*  2 */   " ѡ  Ҫ     ",
/*  3 */   "   Ҫ      ",
/*  4 */   " ѡ  Ҫ     ",
/*  5 */   "       · ",
/*  6 */   " ѡ ȡ Ҫ ɾ    ",
/*  7 */   " ѡ ȡ Ҫ     ",
/*  8 */   " ָ     Ŀ  · ",
/*  9 */   " ѡ ȡ Ҫ     ",
/* 10 */   " ָ     Ŀ  · ",
/* 11 */   "ȷҪɾ ",
/* 12 */   "ȷҪ ",
/* 13 */   "ȷҪ ",
};
//
//	The following Japenese messages are translated by _____________.
//
char	szSJISText[][40]={    // Default Japenese SJIS Display Text
/*  0 */   "ّ̧",
/*  1 */   "ިڸ؂",
/*  2 */   "̧ٖ߰I",
/*  3 */   "]̧ٖ",
/*  4 */   "OύX̧ٖI",
/*  5 */   "V̧ٖ",
/*  6 */   "폜̧ٖI",
/*  7 */   "ړ̧ٖI",
/*  8 */   "̧وړިڸذI",
/*  9 */   "̧ٖ߰I",
/* 10 */   "̧ٺ߰ިڸذI",
/* 11 */   "폜 ",
/* 12 */   "ړ ",
/* 13 */   "߰",
};
//
//	The following Korean messages are translated by _____________.
//
char	szKoreanText[][40]={	// Default Korean Display Text
/*  0 */   "File Utilities",
/*  1 */   "Browse Directory Files",
/*  2 */   "Select Single Source File to Copy",
/*  3 */   "Enter the Destination Filename",
/*  4 */   "Select File to Rename",
/*  5 */   "Enter the New Filename",
/*  6 */   "Select Files to Erase",
/*  7 */   "Select Files to Move",
/*  8 */   "Select Directory to Move Files to",
/*  9 */   "Select Files to Copy",
/* 10 */   "Select Directory to Copy Files to",
/* 11 */   "Erase ",
/* 12 */   "Move ",
/* 13 */   "Copy ",
};

char	szAsciiBtn[][16]={	   // Default English Display Text
/*  0 */   "&List files",
/*  1 */   "&Duplicate",
/*  2 */   "&Rename file",
/*  3 */   "&Erase files",
/*  4 */   "&Move files",
/*  5 */   "&Copy files",
/*  6 */   "E&xit",
/*  7 */   "&Help",
/*  8 */   "&Yes",
/*  9 */   "&No",
/* 10 */   "&All",
};

char	szBig5Btn[][16]={	  // Default English Display Text
/*  0 */   "ɮץؿ[&L]",
/*  1 */   "@[&D]",
/*  2 */   "ɦW[&R]",
/*  3 */   "hɧR[&E]",
/*  4 */   "hɷh[&M]",
/*  5 */   "hɫ[&C]",
/*  6 */   "[&X]",
/*  7 */   "[&H]",
/*  8 */   "Tw[&Y]",
/*  9 */   "n[&N]",
/* 10 */   "n[&A]",
};

char	szGB2312Btn[][16]={	    // Default English Display Text
/*  0 */   "Ŀ¼[&L]",
/*  1 */   "һ[&D]",
/*  2 */   "ĵ[&R]",
/*  3 */   "൵ɾ[&E]",
/*  4 */   "൵[&M]",
/*  5 */   "൵[&C]",
/*  6 */   "[&X]",
/*  7 */   "˵[&H]",
/*  8 */   "ȷ[&Y]",
/*  9 */   "Ҫ[&N]",
/* 10 */   "Ҫ[&A]",
};

char	szSJISBtn[][16]={	  // Default English Display Text
/*  0 */   "ؽ[&L]",
/*  1 */   "߰[&D]",
/*  2 */   "̧ٖ̕ύX[&R]",
/*  3 */   "폜[&E]",
/*  4 */   "ړ[&M]",
/*  5 */   "̺߰[&C]",
/*  6 */   "I[&X]",
/*  7 */   "[&H]",
/*  8 */   "&Yes",
/*  9 */   "&No",
/* 10 */   "S[&A]",
};

char	szKoreanBtn[][16]={	    // Default English Display Text
/*  0 */   "&List files",
/*  1 */   "&Duplicate",
/*  2 */   "&Rename file",
/*  3 */   "&Erase files",
/*  4 */   "&Move files",
/*  5 */   "&Copy files",
/*  6 */   "E&xit",
/*  7 */   "&Help",
/*  8 */   "&Yes",
/*  9 */   "&No",
/* 10 */   "&All",
};

char	szDosAsciiBtn[][16]={	      // Default English Display Text
/*  0 */   " List files ",
/*  1 */   " Duplicate  ",
/*  2 */   " Rename file",
/*  3 */   " Erase files",
/*  4 */   " Move files ",
/*  5 */   " Copy files ",
/*  6 */   "  Exit  ",
/*  7 */   "  Help  ",
/*  8 */   "  Yes   ",
/*  9 */   "   No   ",
/* 10 */   "  All   ",
};

char	szDosBig5Btn[][16]={	     // Default English Display Text
/*  0 */   " L:ɮץؿ ",
/*  1 */   " D:@ ",
/*  2 */   " R:ɦW ",
/*  3 */   " E:hɧR ",
/*  4 */   " M:hɷh ",
/*  5 */   " C:hɫ ",
/*  6 */   " X: ",
/*  7 */   " H: ",
/*  8 */   " Y:Tw ",
/*  9 */   " N:n ",
/* 10 */   " A:n ",
};

char	szDosGB2312Btn[][16]={	       // Default English Display Text
/*  0 */   " L:Ŀ¼ ",
/*  1 */   " D:һ ",
/*  2 */   " R:ĵ ",
/*  3 */   " E:൵ɾ ",
/*  4 */   " M:൵ ",
/*  5 */   " C:൵ ",
/*  6 */   " X: ",
/*  7 */   " H:˵ ",
/*  8 */   " Y:ȷ ",
/*  9 */   " N:Ҫ ",
/* 10 */   " A:Ҫ ",
};

char	szDosSJISBtn[][16]={	     // Default English Display Text
/*  0 */   "L:̧",
/*  1 */   "D:P̧ق̺߰",
/*  2 */   "R:̧ٖ̕ύX",
/*  3 */   "E:̧ق̍폜",
/*  4 */   "M:̧ق̈ړ",
/*  5 */   "C:̧ق̺߰",
/*  6 */   "E:I",
/*  7 */   "H:",
/*  8 */   "Y:Yes",
/*  9 */   "N:No",
/* 10 */   "A:S",
};

char	szDosKoreanBtn[][16]={	       // Default English Display Text
/*  0 */   " List files ",
/*  1 */   " Duplicate  ",
/*  2 */   " Rename file",
/*  3 */   " Erase files",
/*  4 */   " Move files ",
/*  5 */   " Copy files ",
/*  6 */   "  Exit  ",
/*  7 */   "  Help  ",
/*  8 */   "  Yes   ",
/*  9 */   "   No   ",
/* 10 */   "  All   ",
};

#define BIG5		0x404
#define GB2312		0x804
#define SJIS		0x411
#define KOREAN		0x412
#define ASCII		0x409

#define TGBIG5		1
#define TGGB2312	2
#define TGSJIS		5
#define TGKOREAN	3

int	SetMsgText()
{
	char	szBuf[80];
	int	switched;

	IsWin = 1;
	switch(1) {  // Test if we are in TwinCAD ...
	  case !identify("os_type"):
	  case (os_type()<0):	// Must be in DOS version!
		IsWin = 0;
		break;
	}

	setdata(szText,szAsciiText);
	if ( IsWin ) {
	    szBuf = get_profile("General Setup","AlternateLanguage");
	    switched = 0;
	    if ( szBuf!="" )
		switched = eval(szBuf);
	    setdata(szBtn,szAsciiBtn);
	    if ( switched ) {
		switch(os_language()) {
		  case BIG5:	setdata(szText,szBig5Text);	break;
		  case GB2312:	setdata(szText,szGB2312Text);	break;
		  case SJIS:	setdata(szText,szSJISText);	break;
		  case KOREAN:	setdata(szText,szKoreanText);	break;
		}
		switch(os_language()) {
		  case BIG5:	setdata(szBtn,szBig5Btn);     break;
		  case GB2312:	setdata(szBtn,szGB2312Btn);   break;
		  case SJIS:	setdata(szBtn,szSJISBtn);     break;
		  case KOREAN:	setdata(szBtn,szKoreanBtn);   break;
		}
	    }
	}
	else {	// Switched or not, depends on TGF type in DOS version
	    setdata(szBtn,szDosAsciiBtn);
	    switch(@tgftype) {
	      case TGBIG5:    setdata(szText,szBig5Text);     break;
	      case TGGB2312:  setdata(szText,szGB2312Text);   break;
	      case TGSJIS:    setdata(szText,szSJISText);     break;
	      case TGKOREAN:  setdata(szText,szKoreanText);   break;
	    }
	    switch(@tgftype) {
	      case TGBIG5:    setdata(szBtn,szDosBig5Btn);     break;
	      case TGGB2312:  setdata(szBtn,szDosGB2312Btn);   break;
	      case TGSJIS:    setdata(szBtn,szDosSJISBtn);     break;
	      case TGKOREAN:  setdata(szBtn,szDosKoreanBtn);   break;
	    }
	}
	return 0;
}

